@@ -28,7 +28,7 @@ from packages import PackageObject, RpmBase, YumAvailablePackage, parsePackages
 import Errors
 import misc
 
-from sqlutils import executeSQL
+from sqlutils import executeSQL, sql_esc, sql_esc_glob
 import rpmUtils.miscutils
 import sqlutils
 import constants
@@ -37,6 +37,7 @@ from yum.misc import seq_max_split
 from yum.i18n import to_utf8, to_unicode
 import sys
 import re
+import warnings
 
 def catchSqliteException(func):
     """This decorator converts sqlite exceptions into RepoError"""
@@ -258,9 +259,7 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
         if varname.startswith('__') and varname.endswith('__'):
             raise AttributeError, varname
         
-        dbname = varname
-        if db2simplemap.has_key(varname):
-            dbname = db2simplemap[varname]
+        dbname = db2simplemap.get(varname, varname)
         try:
             r = self._sql_MD('primary',
                          "SELECT %s FROM packages WHERE pkgId = ?" % dbname,
@@ -316,7 +315,7 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
     def _loadChangelog(self):
         result = []
         if not self._changelog:
-            if not self.sack.otherdb.has_key(self.repo):
+            if self.repo not in self.sack.otherdb:
                 try:
                     self.sack.populate(self.repo, mdtype='otherdata')
                 except Errors.RepoError:
@@ -330,13 +329,19 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
             # Check count(pkgId) here, the same way we do in searchFiles()?
             # Failure mode is much less of a problem.
             for ob in cur:
-                c_date = ob['date']
+                # Note: Atm. rpm only does days, where (60 * 60 * 24) == 86400
+                #       and we have the hack in _dump_changelog() to keep the
+                #       order the same, so this is a quick way to get rid of
+                #       any extra "seconds".
+                #       We still leak the seconds if there are 100 updates in
+                #       a day ... but don't do that. It also breaks if rpm ever
+                #       gets fixed (but that is unlikely).
+                c_date = 100 * (ob['date'] / 100)
                 c_author = to_utf8(ob['author'])
                 c_log = to_utf8(ob['changelog'])
                 result.append((c_date, _share_data(c_author), c_log))
             self._changelog = result
             return
-    
         
     def returnIdSum(self):
         return (self.checksum_type, self.pkgId)
@@ -345,15 +350,32 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
         self._loadChangelog()
         return self._changelog
     
-    def returnFileEntries(self, ftype='file'):
+    def returnFileEntries(self, ftype='file', primary_only=False):
+        """return list of files based on type, you can pass primary_only=True
+           to limit to those files in the primary repodata"""
+        if primary_only and not self._loadedfiles:
+            sql = "SELECT name as fname FROM files WHERE pkgKey = ? and type = ?"
+            cur = self._sql_MD('primary', sql, (self.pkgKey, ftype))
+            return map(lambda x: x['fname'], cur)
+
         self._loadFiles()
-        return RpmBase.returnFileEntries(self,ftype)
+        return RpmBase.returnFileEntries(self,ftype,primary_only)
     
-    def returnFileTypes(self):
+    def returnFileTypes(self, primary_only=False):
+        """return list of types of files in the package, you can pass
+           primary_only=True to limit to those files in the primary repodata"""
+        if primary_only and not self._loadedfiles:
+            sql = "SELECT DISTINCT type as ftype FROM files WHERE pkgKey = ?"
+            cur = self._sql_MD('primary', sql, (self.pkgKey,))
+            return map(lambda x: x['ftype'], cur)
+
         self._loadFiles()
         return RpmBase.returnFileTypes(self)
 
     def simpleFiles(self, ftype='file'):
+        warnings.warn('simpleFiles() will go away in a future version of Yum.'
+                      'Use returnFileEntries(primary_only=True)\n',
+                      Errors.YumDeprecationWarning, stacklevel=2)
         sql = "SELECT name as fname FROM files WHERE pkgKey = ? and type = ?"
         cur = self._sql_MD('primary', sql, (self.pkgKey, ftype))
         return map(lambda x: x['fname'], cur)
@@ -366,6 +388,8 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
             cur = self._sql_MD('primary', sql, (self.pkgKey,))
             self.prco[prcotype] = [ ]
             for ob in cur:
+                if not ob['name']:
+                    continue
                 prco_set = (_share_data(ob['name']), _share_data(ob['flags']),
                             (_share_data(ob['epoch']),
                              _share_data(ob['version']),
@@ -413,6 +437,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         self._pkgname2pkgkeys = {}
         self._pkgtup2pkgs = {}
         self._pkgnames_loaded = set()
+        self._pkgmatch_fails = set()
+        self._provmatch_fails = set()
         self._arch_allowed = None
         self._pkgExcluder = []
         self._pkgExcludeIds = {}
@@ -476,6 +502,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         self._key2pkg = {}
         self._pkgname2pkgkeys = {}
         self._pkgnames_loaded = set()
+        self._pkgmatch_fails = set()
+        self._provmatch_fails = set()
         self._pkgtup2pkgs = {}
         self._search_cache = {
             'provides' : { },
@@ -528,7 +556,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     # Because we don't want to remove a package from the database we just
     # add it to the exclude list
     def delPackage(self, obj):
-        if not self.excludes.has_key(obj.repo):
+        if obj.repo not in self.excludes:
             self.excludes[obj.repo] = {}
         self.excludes[obj.repo][obj.pkgId] = 1
         if (obj.repo, obj.pkgKey) in self._exclude_whitelist:
@@ -556,10 +584,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return False
 
     def _pkgKeyExcluded(self, repo, pkgKey):
-        if repo in self._all_excludes:
+        if self._all_excludes and repo in self._all_excludes:
             return True
 
-        return (repo, pkgKey) in self._excludes
+        return self._excludes and (repo, pkgKey) in self._excludes
 
     def _pkgExcludedRKNEVRA(self, repo,pkgKey, n,e,v,r,a):
         ''' Main function to use for "can we use this package" question.
@@ -569,7 +597,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 . Tests addPackageExcluder() calls.
         '''
 
-        if (repo, pkgKey) in self._exclude_whitelist:
+        if self._exclude_whitelist and (repo,pkgKey) in self._exclude_whitelist:
             return False
 
         if self._pkgKeyExcluded(repo, pkgKey):
@@ -627,20 +655,20 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return False
 
     def _pkgExcludedRKT(self, repo,pkgKey, pkgtup):
-        ''' Helper function to call _pkgRKNEVRAExcluded.
+        ''' Helper function to call _pkgExcludedRKNEVRA.
             Takes a repo, pkgKey and a package tuple'''
         (n,a,e,v,r) = pkgtup
         return self._pkgExcludedRKNEVRA(repo, pkgKey, n,e,v,r,a)
 
     def _pkgExcludedRKD(self, repo,pkgKey, data):
-        ''' Helper function to call _pkgRKNEVRAExcluded.
+        ''' Helper function to call _pkgExcludedRKNEVRA.
             Takes a repo, pkgKey and a dict of package data'''
         (n,a,e,v,r) = (data['name'], data['arch'],
                        data['epoch'], data['version'], data['release'])
         return self._pkgExcludedRKNEVRA(repo, pkgKey, n,e,v,r,a)
 
     def _pkgExcluded(self, po):
-        ''' Helper function to call _pkgRKNEVRAExcluded.
+        ''' Helper function to call _pkgExcludedRKNEVRA.
             Takes a package object. '''
         return self._pkgExcludedRKT(po.repo, po.pkgKey, po.pkgtup)
 
@@ -682,16 +710,20 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if excluderid is not None:
             self._pkgExcludeIds[excluderid] = len(self._pkgExcluder)
 
+        self._exclude_whitelist = set()
+        self._pkgobjlist_dirty  = True
+
     def _packageByKey(self, repo, pkgKey, exclude=True):
         """ Lookup a pkg by it's pkgKey, if we don't have it load it """
         # Speed hack, so we don't load the pkg. if the pkgKey is dead.
+        assert exclude
         if exclude and self._pkgKeyExcluded(repo, pkgKey):
             return None
 
-        if not self._key2pkg.has_key(repo):
+        if repo not in self._key2pkg:
             self._key2pkg[repo] = {}
             self._pkgname2pkgkeys[repo] = {}
-        if not self._key2pkg[repo].has_key(pkgKey):
+        if pkgKey not in self._key2pkg[repo]:
             sql = "SELECT pkgKey, pkgId, name, epoch, version, release, arch " \
                   "FROM packages WHERE pkgKey = ?"
             data = self._sql_MD('primary', repo, sql, (pkgKey,)).fetchone()
@@ -705,10 +737,14 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             self._pkgtup2pkgs.setdefault(po.pkgtup, []).append(po)
             pkgkeys = self._pkgname2pkgkeys[repo].setdefault(data['name'], [])
             pkgkeys.append(pkgKey)
+        elif exclude and self._pkgExcluded(self._key2pkg[repo][pkgKey]):
+            self._delPackageRK(repo, pkgKey)
+            return None
         return self._key2pkg[repo][pkgKey]
         
     def _packageByKeyData(self, repo, pkgKey, data, exclude=True):
         """ Like _packageByKey() but we already have the data for .pc() """
+        assert exclude
         if exclude and self._pkgExcludedRKD(repo, pkgKey, data):
             return None
         if repo not in self._key2pkg:
@@ -727,11 +763,12 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return the pkgtup. """
         if self._pkgExcludedRKD(repo, pkgKey, data):
             return None
-        if repo not in self._key2pkg:
+        prepo = self._key2pkg.get(repo)
+        if prepo is None:
             self._key2pkg[repo] = {}
             self._pkgname2pkgkeys[repo] = {}
-        if data['pkgKey'] in self._key2pkg.get(repo, {}):
-            return self._key2pkg[repo][data['pkgKey']].pkgtup
+        elif data['pkgKey'] in prepo:
+            return prepo[data['pkgKey']].pkgtup
         return (data['name'], data['arch'],
                 data['epoch'], data['version'], data['release'])
 
@@ -751,13 +788,13 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return ret
 
     def addDict(self, repo, datatype, dataobj, callback=None):
-        if self.added.has_key(repo):
+        if repo in self.added:
             if datatype in self.added[repo]:
                 return
         else:
             self.added[repo] = []
 
-        if not self.excludes.has_key(repo): 
+        if repo not in self.excludes:
             self.excludes[repo] = {}
 
         if dataobj is None:
@@ -796,32 +833,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             pkgs.append(pkg)
         return pkgs
 
-    @staticmethod
-    def _sql_esc(pattern):
-        """ Apply SQLite escaping, if needed. Returns pattern and esc. """
-        esc = ''
-        if "_" in pattern or "%" in pattern:
-            esc = ' ESCAPE "!"'
-            pattern = pattern.replace("!", "!!")
-            pattern = pattern.replace("%", "!%")
-            pattern = pattern.replace("_", "!_")
-        return (pattern, esc)
-
-    def _sql_esc_glob(self, patterns):
-        """ Converts patterns to SQL LIKE format, if required (or gives up if
-            not possible). """
-        ret = []
-        for pattern in patterns:
-            if '[' in pattern: # LIKE only has % and _, so [abc] can't be done.
-                return []      # So Load everything
-
-            # Convert to SQL LIKE format
-            (pattern, esc) = self._sql_esc(pattern)
-            pattern = pattern.replace("*", "%")
-            pattern = pattern.replace("?", "_")
-            ret.append((pattern, esc))
-        return ret
-
     def _skip_all(self):
         """ Are we going to skip every package in all our repos? """
         skip_all = True
@@ -834,6 +845,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     @catchSqliteException
     def _search_primary_files(self, name):
         querytype = 'glob'
+        name = os.path.normpath(name)
         if not misc.re_glob(name):
             querytype = '='        
         results = []
@@ -847,6 +859,32 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         return misc.unique(results)
         
+    @catchSqliteException
+    def _have_fastReturnFileEntries(self):
+        """ Return true if pkg.returnFileEntries(primary_only=True) is fast.
+            basically does "CREATE INDEX pkgfiles ON files (pkgKey);" exist. """
+
+        for (rep,cache) in self.primarydb.items():
+            if rep in self._all_excludes:
+                continue
+            cur = cache.cursor()
+            executeSQL(cur, "PRAGMA index_info(pkgfiles)")
+            #  If we get anything, we're fine. There might be a better way of
+            # saying "anything" but this works.
+            for ob in cur:
+                break
+            else:
+                return False
+
+        return True
+
+    def have_fastReturnFileEntries(self):
+        """ Is calling pkg.returnFileEntries(primary_only=True) faster than
+            using searchFiles(). """
+        if not hasattr(self, '_cached_fRFE'):
+            self._cached_fRFE = self._have_fastReturnFileEntries()
+        return self._cached_fRFE
+
     @catchSqliteException
     def searchFiles(self, name, strict=False):
         """search primary if file will be in there, if not, search filelists, use globs, if possible"""
@@ -861,6 +899,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         glob = True
         file_glob = True
         querytype = 'glob'
+        name = os.path.normpath(name)
         dirname  = os.path.dirname(name)
         filename = os.path.basename(name)
         if strict or not misc.re_glob(name):
@@ -893,20 +932,24 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         # Check to make sure the DB data matches, this should always pass but
         # we've had weird errors. So check it for a bit.
         for repo in self.filelistsdb:
+            # Only check each repo. once ... the libguestfs check :).
+            if hasattr(repo, '_checked_filelists_pkgs'):
+                continue
             pri_pkgs = self._sql_MD_pkg_num('primary',   repo)
             fil_pkgs = self._sql_MD_pkg_num('filelists', repo)
             if pri_pkgs != fil_pkgs:
                 raise Errors.RepoError
+            repo._checked_filelists_pkgs = True
 
         sql_params = []
         dirname_check = ""
         if not glob:
-            (pattern, esc) = self._sql_esc(filename)
+            (pattern, esc) = sql_esc(filename)
             dirname_check = "dirname = ? and filenames LIKE ? %s and " % esc
             sql_params.append(dirname)
             sql_params.append('%' + pattern + '%')
         elif not file_glob:
-            (pattern, esc) = self._sql_esc(filename)
+            (pattern, esc) = sql_esc(filename)
             dirname_check = "dirname GLOB ? and filenames LIKE ? %s and " % esc
             sql_params.append(dirname)
             sql_params.append('%' + pattern + '%')
@@ -980,7 +1023,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return result
         
         searchstring = searchstring.replace("'", "''")
-        (searchstring, esc) = self._sql_esc(searchstring)
+        (searchstring, esc) = sql_esc(searchstring)
         sql = "select DISTINCT pkgKey from packages where %s like '%%%s%%'%s " % (fields[0], searchstring, esc)
         
         for f in fields[1:]:
@@ -1005,7 +1048,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return result
         
         # NOTE: I can't see any reason not to use this all the time, speed
-        # comparison shows them as baiscally equal.
+        # comparison shows them as basically equal.
         if len(searchstrings) > (constants.PATTERNS_MAX / len(fields)):
             tot = {}
             for searchstring in searchstrings:
@@ -1026,7 +1069,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         
         for s in searchstrings:         
             s = s.replace("'", "''")
-            (s, esc) = self._sql_esc(s)
+            (s, esc) = sql_esc(s)
             sql="select pkgKey,1 AS cumul from packages where %s like '%%%s%%'%s " % (fields[0], s, esc)
             for f in fields[1:]:
                 sql = "%s or %s like '%%%s%%'%s " % (sql, f, s, esc)
@@ -1100,7 +1143,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
             executeSQL(cur, "select * from packages where pkgId in %s" %(pkgid_query,))
-            #executeSQL(cur, "select * from packages where pkgId in %s" %(pkgid_query,))            
             for ob in cur:
                 pkgs.append(ob)
         
@@ -1146,7 +1188,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         prcotype = _share_data(prcotype)
         req      = _share_data(req)
-        if self._search_cache[prcotype].has_key(req):
+        if req in self._search_cache[prcotype]:
             return self._search_cache[prcotype][req]
 
         result = { }
@@ -1205,7 +1247,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return result
 
         if not misc.re_primary_filename(name):
-            # if its not in the primary.xml files
+            # If it is not in the primary.xml files
             # search the files.xml file info
             for pkg in self.searchFiles(name, strict=True):
                 result[pkg] = [(name, None, None)]
@@ -1227,7 +1269,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return self._search("requires", name, flags, version)
 
     @catchSqliteException
-    def searchNames(self, names=[]):
+    def searchNames(self, names=[], return_pkgtups=False):
         """return a list of packages matching any of the given names. This is 
            only a match on package name, nothing else"""
         
@@ -1239,11 +1281,16 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         user_names = set(names)
         names = []
         for pkgname in user_names:
+            if pkgname in self._pkgmatch_fails:
+                continue
+
             if loaded_all_names or pkgname in self._pkgnames_loaded:
                 returnList.extend(self._packagesByName(pkgname))
             else:
                 names.append(pkgname)
 
+        if return_pkgtups:
+            returnList = [pkg.pkgtup for pkg in returnList]
         if not names:
             return returnList
 
@@ -1251,7 +1298,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if len(names) > max_entries:
             # Unique is done at user_names time, above.
             for names in seq_max_split(names, max_entries):
-                returnList.extend(self.searchNames(names))
+                returnList.extend(self.searchNames(names, return_pkgtups))
             return returnList
 
         pat_sqls = []
@@ -1265,99 +1312,95 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             cur = cache.cursor()
             executeSQL(cur, qsql, names)
 
+            if return_pkgtups:
+                for ob in cur:
+                    pkgtup = self._pkgtupByKeyData(repo, ob['pkgKey'], ob)
+                    if pkgtup is None:
+                        continue
+                    returnList.append(pkgtup)
+                continue
+
             self._sql_pkgKey2po(repo, cur, returnList, have_data=True)
 
-        # Mark all the processed pkgnames as fully loaded
-        self._pkgnames_loaded.update([name for name in names])
+        if not return_pkgtups:
+            # Mark all the processed pkgnames as fully loaded
+            self._pkgnames_loaded.update([name for name in names])
 
         return returnList
  
     @catchSqliteException
     def searchPrco(self, name, prcotype):
-        """return list of packages having prcotype name (any evr and flag)"""
+        """return list of packages matching name and prcotype """
+        # we take name to be a string of some kind
+        # we parse the string to see if it is a foo > 1.1 or if it is just 'foo'
+        # or what - so we can answer correctly
+        
         if self._skip_all():
             return []
-        
-        name = to_unicode(name)
+        try:
+            (n,f,(e,v,r)) = misc.string_to_prco_tuple(name)
+        except Errors.MiscError, e:
+            raise Errors.PackageSackError, to_unicode(e)
+
+        # The _b means this is a byte string
+        # The _u means this is a unicode string
+        # A bare n is used when, it's unicode but hasn't been evaluated
+        # whether that's actually the right thing to do
+        n_b = n
+        n_u = to_unicode(n)
+        n = n_u
+
         glob = True
         querytype = 'glob'
-        if not misc.re_glob(name):
+        if not misc.re_glob(n):
             glob = False
             querytype = '='
 
+        basic_results = []
         results = []
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            executeSQL(cur, "select DISTINCT pkgKey from %s where name %s ?" % (prcotype,querytype), (name,))
-            self._sql_pkgKey2po(rep, cur, results)
+            executeSQL(cur, "select DISTINCT pkgKey from %s where name %s ?" % (prcotype,querytype), (n,))
+            self._sql_pkgKey2po(rep, cur, basic_results)
         
+        # now we have a list of items matching just the name - let's match them out
+        for po in basic_results:
+            if misc.re_filename(n) and v is None:
+                # file dep add all matches to the results
+                results.append(po)
+                continue
+
+            if not glob:
+                if po.checkPrco(prcotype, (n_b, f, (e,v,r))):
+                    results.append(po)
+            else:
+                # if it is a glob we can't really get any closer to checking it
+                results.append(po)
         # If it's not a provides or a filename, we are done
         if prcotype != "provides":
             return results
-        if not misc.re_filename(name):
+        if not misc.re_filename(n):
             return results
 
         # If it is a filename, search the primary.xml file info
-        results.extend(self._search_primary_files(name))
+        results.extend(self._search_primary_files(n))
 
-        # if its in the primary.xml files then skip the other check
-        if misc.re_primary_filename(name) and not glob:
+        # If it is in the primary.xml files then skip the other check
+        if misc.re_primary_filename(n) and not glob:
             return misc.unique(results)
 
         # If it is a filename, search the files.xml file info
-        results.extend(self.searchFiles(name))
+        results.extend(self.searchFiles(n))
         return misc.unique(results)
-        
-        
-        #~ #FIXME - comment this all out below here
-        #~ for (rep,cache) in self.filelistsdb.items():
-            #~ cur = cache.cursor()
-            #~ (dirname,filename) = os.path.split(name)
-            #~ # FIXME: why doesn't this work???
-            #~ if 0: # name.find('%') == -1: # no %'s in the thing safe to LIKE
-                #~ executeSQL(cur, "select packages.pkgId as pkgId,\
-                    #~ filelist.dirname as dirname,\
-                    #~ filelist.filetypes as filetypes,\
-                    #~ filelist.filenames as filenames \
-                    #~ from packages,filelist where \
-                    #~ (filelist.dirname LIKE ? \
-                    #~ OR (filelist.dirname LIKE ? AND\
-                    #~ filelist.filenames LIKE ?))\
-                    #~ AND (filelist.pkgKey = packages.pkgKey)", (name,dirname,filename))
-            #~ else: 
-                #~ executeSQL(cur, "select packages.pkgId as pkgId,\
-                    #~ filelist.dirname as dirname,\
-                    #~ filelist.filetypes as filetypes,\
-                    #~ filelist.filenames as filenames \
-                    #~ from filelist,packages where dirname = ? AND filelist.pkgKey = packages.pkgKey" , (dirname,))
-
-            #~ matching_ids = []
-            #~ for res in cur:
-                #~ if self._excluded(rep, res['pkgId']):
-                    #~ continue
-                
-                #~ #FIXME - optimize the look up here by checking for single-entry filenames
-                #~ quicklookup = {}
-                #~ for fn in decodefilenamelist(res['filenames']):
-                    #~ quicklookup[fn] = 1
-                
-                #~ # If it matches the dirname, that doesnt mean it matches
-                #~ # the filename, check if it does
-                #~ if filename and not quicklookup.has_key(filename):
-                    #~ continue
-                
-                #~ matching_ids.append(str(res['pkgId']))
-                
-            
-            #~ pkgs = self._getListofPackageDetails(matching_ids)
-            #~ for pkg in pkgs:
-                #~ results.append(self.pc(rep,pkg))
-        
-        #~ return results
 
     def searchProvides(self, name):
         """return list of packages providing name (any evr and flag)"""
-        return self.searchPrco(name, "provides")
+        if name in self._provmatch_fails:
+            return []
+        ret = self.searchPrco(name, "provides")
+        if not ret:
+            self._provmatch_fails.add(name)
+        return ret
                 
     def searchRequires(self, name):
         """return list of packages requiring name (any evr and flag)"""
@@ -1420,6 +1463,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
     @catchSqliteException
     def returnNewestByName(self, name=None, patterns=None, ignore_case=False):
+        """return list of newest packages based on name matching
+           this means(in name.arch form): foo.i386 and foo.noarch will
+           be compared to each other for highest version.
+           Note that given: foo-1.i386; foo-2.i386 and foo-3.x86_64
+           The last _two_ pkgs will be returned, not just one of them. """
         # If name is set do it from the database otherwise use our parent's
         # returnNewestByName
         if self._skip_all():
@@ -1485,7 +1533,8 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                   'sql_envra', 'sql_nevra']
         need_full = False
         for pat in patterns:
-            if misc.re_full_search_needed(pat):
+            if (misc.re_full_search_needed(pat) and
+                (ignore_case or pat not in self._pkgnames_loaded)):
                 need_full = True
                 break
 
@@ -1496,7 +1545,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if len(patterns) > pat_max:
             patterns = []
         if ignore_case:
-            patterns = self._sql_esc_glob(patterns)
+            patterns = sql_esc_glob(patterns)
         else:
             tmp = []
             need_glob = False
@@ -1516,24 +1565,30 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         """Yields all the package data for the given params. Excludes are done
            at this stage. """
 
+        pat_sqls = []
+        pat_data = []
+        for (pattern, rest) in patterns:
+            if not ignore_case and pattern in self._pkgmatch_fails:
+                continue
+
+            for field in fields:
+                if ignore_case:
+                    pat_sqls.append("%s LIKE ?%s" % (field, rest))
+                else:
+                    pat_sqls.append("%s %s ?" % (field, rest))
+                pat_data.append(pattern)
+        if patterns and not pat_sqls:
+            return
+
+        if pat_sqls:
+            qsql = _FULL_PARSE_QUERY_BEG + " OR ".join(pat_sqls)
+        else:
+            qsql = """select pkgId, pkgKey, name,epoch,version,release,arch
+                      from packages"""
+
         for (repo,cache) in self.primarydb.items():
             if (repoid == None or repoid == repo.id):
                 cur = cache.cursor()
-
-                qsql = """select pkgId, pkgKey, name,epoch,version,release,arch 
-                          from packages"""
-
-                pat_sqls = []
-                pat_data = []
-                for (pattern, rest) in patterns:
-                    for field in fields:
-                        if ignore_case:
-                            pat_sqls.append("%s LIKE ?%s" % (field, rest))
-                        else:
-                            pat_sqls.append("%s %s ?" % (field, rest))
-                        pat_data.append(pattern)
-                if pat_sqls:
-                    qsql = _FULL_PARSE_QUERY_BEG + " OR ".join(pat_sqls)
                 executeSQL(cur, qsql, pat_data)
                 for x in cur:
                     yield (repo, x)
@@ -1551,6 +1606,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
 
         for (repo, x) in self._yieldSQLDataList(repoid, patterns, fields,
                                                 ignore_case):
+            # Can't use: _sql_pkgKey2po because we change repos.
             po = self._packageByKeyData(repo, x['pkgKey'], x)
             if po is None:
                 continue
@@ -1561,6 +1617,17 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if not need_full and repoid is None:
             # Mark all the processed pkgnames as fully loaded
             self._pkgnames_loaded.update([po.name for po in returnList])
+        if need_full:
+            for (pat, rest) in patterns:
+                if rest not in ('=', ''): # Wildcards: 'glob' or ' ESCAPE "!"'
+                    continue
+                for pkg in returnList:
+                    if pkg.name == pat:
+                        self._pkgnames_loaded.add(pkg.name)
+                        break
+        if not returnList:
+            for (pat, rest) in patterns:
+                self._pkgmatch_fails.add(pat)
 
         return returnList
                 
@@ -1617,6 +1684,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         # Haven't loaded everything, so _just_ get the pkgtups...
         data = self._setupPkgObjList(repoid, patterns, ignore_case)
         (need_full, patterns, fields, names) = data
+        if names:
+            return [pkg.pkgtup for pkg in self.searchNames(patterns)]
+
         for (repo, x) in self._yieldSQLDataList(repoid, patterns, fields,
                                                 ignore_case):
             # NOTE: Can't unexclude things...
