@@ -1,3 +1,4 @@
+#! /usr/bin/python -tt
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -26,11 +27,48 @@ to rpm.
 """
 
 from constants import *
-from packageSack import PackageSack
+from packageSack import PackageSack, PackageSackVersion
 from packages import YumInstalledPackage
 from sqlitesack import YumAvailablePackageSqlite
 import Errors
 import warnings
+import misc
+
+class GetProvReqOnlyPackageSack(PackageSack):
+    def __init__(self, need_files=False):
+        PackageSack.__init__(self)
+        self._need_index_files = need_files
+
+    def __addPackageToIndex_primary_files(self, obj):
+        for ftype in obj.returnFileTypes(primary_only=True):
+            for file in obj.returnFileEntries(ftype, primary_only=True):
+                self._addToDictAsList(self.filenames, file, obj)
+    def __addPackageToIndex_files(self, obj):
+        for ftype in obj.returnFileTypes():
+            for file in obj.returnFileEntries(ftype):
+                self._addToDictAsList(self.filenames, file, obj)
+    def _addPackageToIndex(self, obj):
+        for (n, fl, (e,v,r)) in obj.returnPrco('provides'):
+            self._addToDictAsList(self.provides, n, obj)
+        for (n, fl, (e,v,r)) in obj.returnPrco('requires'):
+            self._addToDictAsList(self.requires, n, obj)
+        if self._need_index_files:
+            self.__addPackageToIndex_files(obj)
+        else:
+            self.__addPackageToIndex_primary_files(obj)
+
+    def __buildFileIndexes(self):
+        for repoid in self.pkgsByRepo:
+            for obj in self.pkgsByRepo[repoid]:
+                self.__addPackageToIndex_files(obj)
+    def searchFiles(self, name):
+        if not self._need_index_files and not misc.re_primary_filename(name):
+            self._need_index_files = True
+            if self.indexesBuilt:
+                self.filenames = {}
+                self.__buildFileIndexes()
+
+        return PackageSack.searchFiles(self, name)
 
 class TransactionData:
     """Data Structure designed to hold information on a yum Transaction Set"""
@@ -45,13 +83,14 @@ class TransactionData:
         self.debug = 0
         self.changed = False
         self.installonlypkgs = []
-        
+        self.state_counter = 0
         self.conditionals = {} # key = pkgname, val = list of pos to add
 
         self.rpmdb = None
         self.pkgSack = None
         self.pkgSackPackages = 0
         self.localSack = PackageSack()
+        self._inSack = GetProvReqOnlyPackageSack()
 
         # lists of txmbrs in their states - just placeholders
         self.instgroups = []
@@ -65,6 +104,7 @@ class TransactionData:
         self.depupdated = []
         self.reinstalled = []
         self.downgraded = []
+        self.failed = []
         
     def __len__(self):
         return len(self.pkgdict)
@@ -92,7 +132,7 @@ class TransactionData:
         if pkgtup is None:
             for members in self.pkgdict.itervalues():
                 returnlist.extend(members)            
-        elif self.pkgdict.has_key(pkgtup):
+        elif pkgtup in self.pkgdict:
             returnlist.extend(self.pkgdict[pkgtup])
         return returnlist
             
@@ -155,6 +195,39 @@ class TransactionData:
 
         return result
 
+    def deselect(self, pattern):
+        """ Remove these packages from the transaction. This is more user
+            orientated than .remove(). Used from kickstart/install -blah. """
+
+        #  We don't have a returnPackages() here, so just try the "simple" 
+        # specifications. Pretty much 100% hit rate on kickstart.
+        txmbrs = self.matchNaevr(pattern)
+        if not txmbrs:
+            na = pattern.rsplit('.', 2)
+            if len(na) == 2:
+                txmbrs = self.matchNaevr(na[0], na[1])
+
+        if not txmbrs:
+            if self.pkgSack is None:
+                pkgs = []
+            else:
+                pkgs = self.pkgSack.returnPackages(patterns=[pattern])
+            if not pkgs:
+                pkgs = self.rpmdb.returnPackages(patterns=[pattern])
+
+            for pkg in pkgs:
+                txmbrs.extend(self.getMembers(pkg.pkgtup))
+                #  Now we need to do conditional group packages, so they don't
+                # get added later on. This is hacky :(
+                for req, cpkgs in self.conditionals.iteritems():
+                    if pkg in cpkgs:
+                        cpkgs.remove(pkg)
+                        self.conditionals[req] = cpkgs
+
+        for txmbr in txmbrs:
+            self.remove(txmbr.pkgtup)
+        return txmbrs
+
     def _isLocalPackage(self, txmember):
         # Is this the right criteria?
         # FIXME: This is kinda weird, we really want all local pkgs to be in a
@@ -183,7 +256,7 @@ class TransactionData:
         for oldpo in txmember.updates:
             self.addUpdated(oldpo, txmember.po)
 
-        if not self.pkgdict.has_key(txmember.pkgtup):
+        if txmember.pkgtup not in self.pkgdict:
             self.pkgdict[txmember.pkgtup] = []
         else:
             self.debugprint("Package: %s.%s - %s:%s-%s already in ts" % txmember.pkgtup)
@@ -194,12 +267,19 @@ class TransactionData:
         self.pkgdict[txmember.pkgtup].append(txmember)
         self._namedict.setdefault(txmember.name, []).append(txmember)
         self.changed = True
+        self.state_counter += 1
         if self._isLocalPackage(txmember):
             self.localSack.addPackage(txmember.po)
         elif isinstance(txmember.po, YumAvailablePackageSqlite):
             self.pkgSackPackages += 1
+        if self._inSack is not None and txmember.output_state in TS_INSTALL_STATES:
+            if not txmember.po.have_fastReturnFileEntries():
+                # In theory we could keep this on if a "small" repo. fails
+                self._inSack = None
+            else:
+                self._inSack.addPackage(txmember.po)
 
-        if self.conditionals.has_key(txmember.name):
+        if txmember.name in self.conditionals:
             for pkg in self.conditionals[txmember.name]:
                 if self.rpmdb.contains(po=pkg):
                     continue
@@ -210,7 +290,7 @@ class TransactionData:
 
     def remove(self, pkgtup):
         """remove a package from the transaction"""
-        if not self.pkgdict.has_key(pkgtup):
+        if pkgtup not in self.pkgdict:
             self.debugprint("Package: %s not in ts" %(pkgtup,))
             return
         for txmbr in self.pkgdict[pkgtup]:
@@ -219,6 +299,8 @@ class TransactionData:
                 self.localSack.delPackage(txmbr.po)
             elif isinstance(txmbr.po, YumAvailablePackageSqlite):
                 self.pkgSackPackages -= 1
+            if self._inSack is not None and txmbr.output_state in TS_INSTALL_STATES:
+                self._inSack.delPackage(txmbr.po)
             self._namedict[txmbr.name].remove(txmbr)
             self._unresolvedMembers.add(txmbr)
         
@@ -226,10 +308,11 @@ class TransactionData:
         if not self._namedict[pkgtup[0]]:
             del self._namedict[pkgtup[0]]
         self.changed = True        
+        self.state_counter += 1
     
     def exists(self, pkgtup):
         """tells if the pkg is in the class"""
-        if self.pkgdict.has_key(pkgtup):
+        if pkgtup in self.pkgdict:
             if len(self.pkgdict[pkgtup]) != 0:
                 return 1
         
@@ -270,8 +353,7 @@ class TransactionData:
                     self.updated.append(txmbr)
                     
             elif txmbr.output_state in (TS_INSTALL, TS_TRUEINSTALL):
-                if include_reinstall and self.rpmdb.contains(po=txmbr.po):
-                    txmbr.reinstall = True
+                if include_reinstall and txmbr.reinstall:
                     self.reinstalled.append(txmbr)
                     continue
 
@@ -324,7 +406,6 @@ class TransactionData:
         self.downgraded.sort()
         self.failed.sort()
 
-    
     def addInstall(self, po):
         """adds a package as an install but in mode 'u' to the ts
            takes a packages object and returns a TransactionMember Object"""
@@ -338,6 +419,11 @@ class TransactionData:
         txmbr.po.state = TS_INSTALL        
         txmbr.ts_state = 'u'
         txmbr.reason = 'user'
+
+        if self.rpmdb.contains(po=txmbr.po):
+            txmbr.reinstall = True
+        
+        self.findObsoletedByThisMember(txmbr)
         self.add(txmbr)
         return txmbr
 
@@ -351,6 +437,10 @@ class TransactionData:
         txmbr.po.state = TS_INSTALL        
         txmbr.ts_state = 'i'
         txmbr.reason = 'user'
+
+        if self.rpmdb.contains(po=txmbr.po):
+            txmbr.reinstall = True
+
         self.add(txmbr)
         return txmbr
     
@@ -384,6 +474,7 @@ class TransactionData:
             txmbr.updates.append(oldpo)
             
         self.add(txmbr)
+        self.findObsoletedByThisMember(txmbr)
         return txmbr
 
     def addDowngrade(self, po, oldpo):
@@ -411,7 +502,7 @@ class TransactionData:
         txmbr.current_state = TS_INSTALL
         txmbr.output_state =  TS_UPDATED
         txmbr.po.state = TS_UPDATED
-        txmbr.ts_state = None # FIXME: should use a real state here.
+        txmbr.ts_state = 'ud'
         txmbr.relatedto.append((updating_po, 'updatedby'))
         txmbr.updated_by.append(updating_po)
         self.add(txmbr)
@@ -428,6 +519,10 @@ class TransactionData:
         txmbr.ts_state = 'u'
         txmbr.relatedto.append((oldpo, 'obsoletes'))
         txmbr.obsoletes.append(oldpo)
+
+        if self.rpmdb.contains(po=txmbr.po):
+            txmbr.reinstall = True
+
         self.add(txmbr)
         return txmbr
 
@@ -439,10 +534,16 @@ class TransactionData:
         txmbr.current_state = TS_INSTALL
         txmbr.output_state =  TS_OBSOLETED
         txmbr.po.state = TS_OBSOLETED
-        txmbr.ts_state = None # FIXME: should use a real state here.
+        txmbr.ts_state = 'od'
         txmbr.relatedto.append((obsoleting_po, 'obsoletedby'))
         txmbr.obsoleted_by.append(obsoleting_po)
         self.add(txmbr)
+        for otxmbr in self.getMembersWithState(obsoleting_po.pkgtup,
+                                               [TS_OBSOLETING]):
+            if po in otxmbr.obsoletes:
+                continue
+            otxmbr.relatedto.append((po, 'obsoletes'))
+            otxmbr.obsoletes.append(po)
         return txmbr
 
 
@@ -454,10 +555,15 @@ class TransactionData:
         """return dict { packages -> list of matching provides }
         searches in packages to be installed"""
         result = { }
-        if self.pkgSackPackages:
+        if not self.pkgSackPackages:
+            pass
+        elif self._inSack is None:
             for pkg, hits in self.pkgSack.getProvides(name, flag, version).iteritems():
                 if self.getMembersWithState(pkg.pkgtup, TS_INSTALL_STATES):
                     result[pkg] = hits
+        else:
+            for pkg, hits in self._inSack.getProvides(name, flag, version).iteritems():
+                result[pkg] = hits
         result.update(self.localSack.getProvides(name, flag, version))
         return result
 
@@ -480,10 +586,16 @@ class TransactionData:
         """return dict { packages -> list of matching provides }
         searches in packages to be installed"""
         result = { }
-        if self.pkgSackPackages:
+        if not self.pkgSackPackages:
+            pass
+        elif self._inSack is None:
             for pkg, hits in self.pkgSack.getRequires(name, flag, version).iteritems():
                 if self.getMembersWithState(pkg.pkgtup, TS_INSTALL_STATES):
                     result[pkg] = hits
+        else:
+            for pkg, hits in self._inSack.getRequires(name, flag, version).iteritems():
+                result[pkg] = hits
+
         result.update(self.localSack.getRequires(name, flag, version))
         return result
 
@@ -503,6 +615,62 @@ class TransactionData:
         result.update(self.getNewRequires(name, flag, version))
         return result
 
+    def futureRpmDBVersion(self):
+        """ Return a simple version for the future rpmdb. Works like
+            rpmdb.simpleVersion(main_only=True)[0], but for the state the rpmdb
+            will be in after the transaction. """
+        pkgs = self.rpmdb.returnPackages()
+        _reinstalled_pkgtups = {}
+        for txmbr in self.getMembersWithState(None, TS_INSTALL_STATES):
+            # reinstalls have to use their "new" checksum data, in case it's
+            # different.
+            if txmbr.reinstall:
+                _reinstalled_pkgtups[txmbr.po.pkgtup] = txmbr.po
+            pkgs.append(txmbr.po)
+
+        self.rpmdb.preloadPackageChecksums()
+        main = PackageSackVersion()
+        pkg_checksum_tups = []
+        for pkg in sorted(pkgs):
+            if pkg.repoid != 'installed':
+                # Paste from PackageSackBase.simpleVersion()
+                csum = pkg.returnIdSum()
+                main.update(pkg, csum)
+                pkg_checksum_tups.append((pkg.pkgtup, csum))
+                continue
+
+            # Installed pkg, see if it's about to die
+            if self.getMembersWithState(pkg.pkgtup, TS_REMOVE_STATES):
+                continue
+            # ...or die and be risen again (Zombie!)
+            if pkg.pkgtup in _reinstalled_pkgtups:
+                continue
+
+            # Paste from rpmdb.simpleVersion()
+            ydbi = pkg.yumdb_info
+            csum = None
+            if 'checksum_type' in ydbi and 'checksum_data' in ydbi:
+                csum = (ydbi.checksum_type, ydbi.checksum_data)
+            #  We need all the pkgtups, so we even save the ones without a
+            # checksum.
+            pkg_checksum_tups.append((pkg.pkgtup, csum))
+            main.update(pkg, csum)
+
+        self.rpmdb.transactionCachePackageChecksums(pkg_checksum_tups)
+
+        return main
+    
+    def findObsoletedByThisMember(self, txmbr):
+        """addObsoleted() pkgs for anything that this txmbr will obsolete"""
+        # this is mostly to keep us in-line with what will ACTUALLY happen
+        # when rpm hits the obsoletes, whether we added them or not
+        for obs_n in txmbr.po.obsoletes_names:
+            for pkg in self.rpmdb.searchNevra(name=obs_n):
+                if pkg.obsoletedBy([txmbr.po]):
+                    self.addObsoleted(pkg, txmbr.po)
+                    txmbr.output_state = TS_OBSOLETING
+                    txmbr.po.state = TS_OBSOLETING
+
 class ConditionalTransactionData(TransactionData):
     """A transaction data implementing conditional package addition"""
     def __init__(self):
@@ -560,6 +728,7 @@ class SortableTransactionData(TransactionData):
         self._sorted.reverse()
         return self._sorted
 
+
 class TransactionMember:
     """Class to describe a Transaction Member (a pkg to be installed/
        updated/erased)."""
@@ -572,7 +741,7 @@ class TransactionMember:
         self.output_state = None # what state to list if printing it
         self.isDep = 0
         self.reason = 'user' # reason for it to be in the transaction set
-        self.process = None # 
+        self.process = None #  I think this is used nowhere by nothing - skv 2010/11/03
         self.relatedto = [] # ([relatedpkg, relationship)]
         self.depends_on = []
         self.obsoletes = []
@@ -581,6 +750,7 @@ class TransactionMember:
         self.updated_by = []
         self.downgrades = []
         self.downgraded_by = []
+        self.reinstall = False
         self.groups = [] # groups it's in
         self._poattr = ['pkgtup', 'repoid', 'name', 'arch', 'epoch', 'version',
                         'release']
@@ -589,6 +759,14 @@ class TransactionMember:
             val = getattr(self.po, attr)
             setattr(self, attr, val)
 
+        if po.repoid == 'installed':
+            #  We want to load these so that we can auto hardlink in the same
+            # new values. Because of the hardlinks it should be really cheap
+            # to load them ... although it's still a minor hack.
+            po.yumdb_info.get('from_repo')
+            po.yumdb_info.get('releasever')
+            po.yumdb_info.get('changed_by')
+
     def setAsDep(self, po=None):
         """sets the transaction member as a dependency and maps the dep into the
            relationship list attribute"""
@@ -605,19 +783,47 @@ class TransactionMember:
         return object.__hash__(self)
             
     def __str__(self):
-        return "%s.%s %s-%s-%s - %s" % (self.name, self.arch, self.epoch,
+        return "%s.%s %s:%s-%s - %s" % (self.name, self.arch, self.epoch,
                                         self.version, self.release, self.ts_state)
 
     def __repr__(self):
         return "<%s : %s (%s)>" % (self.__class__.__name__, str(self),hex(id(self))) 
+    
+    def _dump(self):
+        msg = "mbr: %s,%s,%s,%s,%s %s\n" % (self.name, self.arch, self.epoch, 
+                     self.version, self.release, self.current_state)
+        msg += "  repo: %s\n" % self.po.repo.id
+        msg += "  ts_state: %s\n" % self.ts_state
+        msg += "  output_state: %s\n" %  self.output_state
+        msg += "  isDep: %s\n" %  bool(self.isDep)
+        msg += "  reason: %s\n" % self.reason
+        #msg += "  process: %s\n" % self.process
+        msg += "  reinstall: %s\n" % bool(self.reinstall)
         
-    # This is the tricky part - how do we nicely setup all this data w/o going insane
-    # we could make the txmember object be created from a YumPackage base object
-    # we still may need to pass in 'groups', 'ts_state', 'output_state', 'reason', 'current_state'
-    # and any related packages. A world of fun that will be, you betcha
-    
-    
-    # definitions
-    # current and output states are defined in constants
-    # relationships are defined in constants
-    # ts states are: u, i, e
+        if self.relatedto:
+            msg += "  relatedto:"
+            for (po, rel) in self.relatedto:
+                pkgorigin = 'a'
+                if isinstance(po, YumInstalledPackage):
+                    pkgorigin = 'i'
+                msg += " %s,%s,%s,%s,%s@%s:%s" % (po.name, po.arch, po.epoch, 
+                      po.version, po.release, pkgorigin, rel)
+            msg += "\n"
+            
+        for lst in ['depends_on', 'obsoletes', 'obsoleted_by', 'downgrades',
+                    'downgraded_by', 'updates', 'updated_by']:
+            thislist = getattr(self, lst)
+            if thislist:
+                msg += "  %s:" % lst
+                for po in thislist:
+                    pkgorigin = 'a'
+                    if isinstance(po, YumInstalledPackage):
+                        pkgorigin = 'i'
+                    msg += " %s,%s,%s,%s,%s@%s" % (po.name, po.arch, po.epoch, 
+                        po.version, po.release, pkgorigin)
+                msg += "\n"
+                
+        if self.groups:
+            msg += "  groups: %s\n" % ' '.join(self.groups)
+
+        return msg
